@@ -18,15 +18,11 @@ def main():
         command = sys.argv[1]
         sys.argv = sys.argv[1:]
 
-        if command == 'setup':
-
-            return yac.cli.setupdb.main()
-
-        elif command == 'restore':
+        if command == 'restore':
 
             return yac.cli.restoredb.main()
 
         else:
 
-            print "Command not supported or known"
+            print "yac db command not supported or known"
             show_primer(['db','primer'])
\ No newline at end of file
