@@ -1,4 +1,4 @@
-import boto3, jmespath, sys, json, requests
+import boto3, jmespath, sys, json, requests, os, imp
 from sets import Set
 from yac.lib.variables import get_variable, set_variable
 from yac.lib.naming import get_stack_name
@@ -7,14 +7,15 @@ from yac.lib.stack import get_stack_vpc, get_stack_ssh_keys
              
 USER_INPUTS_PARAM_KEY = "UserInputs"
 
-def get_inputs(params):
+def get_inputs(params, inputs_key='service-inputs'):
 
     user_inputs = {}
+    parameter_mapping = {}
 
-    # get service input params
-    if 'service-inputs' in params and 'inputs' in params['service-inputs']:
+    # get input params
+    if inputs_key in params and 'inputs' in params[inputs_key]:
 
-        service_param = params['service-inputs']['inputs']
+        service_param = params[inputs_key]['inputs']
 
         for param_key in service_param:
 
@@ -22,41 +23,75 @@ def get_inputs(params):
             value = handle_service_input(params, service_param[param_key], param_key)
 
             # accumulate this input
-            set_variable(user_inputs,param_key,value,service_param[param_key]['help'])
+            set_variable(user_inputs,param_key,value, service_param[param_key]['help'])
+
+            # if this input needs to be mapped to stack parameter
+            if 'param' in service_param[param_key]:
+                # accumulate this mapping
+                this_mapping = service_param[param_key]['param']
+                this_mapping['value'] = value
+                parameter_mapping[param_key]=this_mapping
 
     # get conditional input params
-    if 'service-inputs' in params and 'conditional-inputs' in params['service-inputs']:
-        conditional_params = params['service-inputs']['conditional-inputs']
+    if inputs_key in params and 'conditional-inputs' in params[inputs_key]:
+        conditional_params = params[inputs_key]['conditional-inputs']
         
         for param_key in conditional_params:
 
-            # get the conditional check
-            conditional_input = conditional_params[param_key]['condition']['input']
+            value = ""
 
-            conditional_value = ""
-            if 'value' in conditional_params[param_key]['condition']:
-                conditional_value = conditional_params[param_key]['condition']['value']
+            if 'input' in conditional_params[param_key]['condition']:
 
-            # get state of the input
-            input_setting = get_variable(params,conditional_input,"")
+                # get the conditional check
+                conditional_input = conditional_params[param_key]['condition']['input']
 
-            # if not conditional value specified, or a value was specified and the 
-            # input setting value matches the condition value, then
-            # perform the conditional check
-            if (not conditional_value or 
-                  (conditional_value and conditional_value == input_setting)
-                ):
+                conditional_value = ""
+                if 'value' in conditional_params[param_key]['condition']:
+                    conditional_value = conditional_params[param_key]['condition']['value']
+
+                # get state of the input
+                conditional_setting = get_variable(params,conditional_input,"")
+
+                # print "input %s, condition setting: %s, condition value: %s"%(param_key,conditional_setting,conditional_value)
+
+            elif 'fxn' in conditional_params[param_key]['condition']:
+
+                # get the conditional check
+                conditional_fxn = str(conditional_params[param_key]['condition']['fxn'])
+
+                conditional_value = ""
+                if 'value' in conditional_params[param_key]['condition']:
+                    conditional_value = conditional_params[param_key]['condition']['value']
+
+                # execute the conditional fxn
+                conditional_setting = eval(conditional_fxn)(params)                
+
+            # if no conditional value specified, or a value was specified and the 
+            # input/fxn value matches the condition value, then
+            # handle the service input (o.w. skip)
+            if ( ('value' not in conditional_params[param_key]['condition']) or 
+                 (conditional_value == conditional_setting) ):
+
+                # print "handling %s ..."%param_key
 
                 # handle this conditional input
                 value = handle_service_input(params, conditional_params[param_key], param_key)
 
                 # accumulate the resulting input
-                set_variable(user_inputs,param_key,value,conditional_params[param_key]['help'])
+                set_variable(user_inputs,param_key,value, conditional_params[param_key]['help'])
 
-    # convert inputs into a string and save to a variable that can be connected to the stack
-    # for reference by 'yac stack' during stack updates
-    user_inputs_str = json.dumps(user_inputs)
-    set_variable(params,'user-inputs',user_inputs_str)
+            # if this input needs to be mapped to stack parameter
+            if 'param' in conditional_params[param_key]:
+                # accumulate this mapping
+                this_mapping = conditional_params[param_key]['param']
+                this_mapping['value'] = value
+                parameter_mapping[param_key]=this_mapping
+
+    # save user_inputs to the params for external acccess
+    set_variable(params,'user-inputs',user_inputs)
+
+    # save parameter_mapping to the params for external acccess
+    set_variable(params,'param-mapping',parameter_mapping)
 
 def handle_service_input(params, dynamic_param, param_key):
 
@@ -101,7 +136,7 @@ def handle_service_input(params, dynamic_param, param_key):
 
     return value
 
-# set a scalar input in params
+# set an input in params
 def set_service_input(params, 
                      param_key, 
                      param_desc,
@@ -113,32 +148,48 @@ def set_service_input(params,
 
     value = ""
 
-    # see if the param is already defined statically or as a vpc-default
-    value = get_variable_or_default(params, param_key)
+    # see if the param is already defined statically
+    value = get_variable(params, param_key,'absolutely-nada')
 
-    if not value:
+    if value == 'absolutely-nada':
 
         # get the name of the stack
         stack_name = get_stack_name(params)
 
-        if not stack_exists(stack_name):
+        # see if a vpc default value exists
+        default_value = get_vpc_default(params, param_key)
 
-            # this is a new stack...
-            # prompt user to provide a value for this param
-            value = eval(wizard_fxn)(param_desc,param_help,options,required,wizard_arg)
+        if stack_exists(stack_name) and not default_value:
 
-        else:
             # a stack for this service already exists ...
+
             # determine the param value currently in use
-            value = get_stack_service_inputs_value(params, [USER_INPUTS_PARAM_KEY, param_key])
+            value = get_stack_service_inputs_value(stack_name, param_key)
  
             # give the user the choice of changing existing value
             if change_wizard(value, param_desc):
                 value = eval(wizard_fxn)(param_desc,param_help,options,required,wizard_arg)
 
-        if value:
+        elif not stack_exists(stack_name) and not default_value:
 
-            set_variable(params, param_key,value, param_desc)
+            # this is a new stack and no default value for this param is available ...
+
+            # prompt user to provide a value for this param
+            value = eval(wizard_fxn)(param_desc,param_help,options,required,wizard_arg)
+
+        elif not stack_exists(stack_name) and default_value:
+
+            # use the default
+            print "setting %s to %s per vpc prefs"%(param_key,default_value)
+            value = default_value
+
+        elif stack_exists(stack_name) and default_value:
+
+            # use the default
+            print "keeping %s as %s per vpc prefs"%(param_key,default_value)
+            value = default_value            
+
+        set_variable(params, param_key,value, param_desc)
 
     return value
 
@@ -162,20 +213,90 @@ def get_variable_or_default(params, param_key):
 
     return value
 
+def does_stack_exist(params):
+
+    stack_name = get_stack_name(params)
+
+    return stack_exists(stack_name)
+
+# get param value set via vpc prefs
+def get_vpc_default(params, param_key):
+
+    # see if the param is defined as a vpc-default via vpc prefs
+    vpc_defaults = get_variable(params,'vpc-defaults', {})
+
+    value = get_variable(vpc_defaults,param_key)
+
+    return value
 
 def get_user_inputs_as_stack_param(service_params):
 
     stack_parameter = {}
 
-    user_inputs = get_variable(service_params, 'user-inputs')
+    # get inputs driven from service file
+    user_inputs = get_user_inputs(service_params)
 
     if user_inputs:
 
+        user_inputs_str = json.dumps(user_inputs)
+
+        print user_inputs_str
+
         stack_parameter['ParameterKey'] = USER_INPUTS_PARAM_KEY
-        stack_parameter['ParameterValue'] = user_inputs
+        stack_parameter['ParameterValue'] = user_inputs_str
 
     return stack_parameter
 
+def get_mapping_as_stack_params(service_params, costing=False):
+
+    stack_parameters = []
+
+    # get mapping that connects user inputs to specific stack parameters
+    param_mapping = get_variable(service_params, 'param-mapping')
+
+    if param_mapping:
+
+        # get the state of the stack
+        stack_exists = get_variable(service_params,'stack-exists',False)
+
+        for param_key in param_mapping:
+
+            # if either:
+            #   1) params are being using for costing (not actual printing), or 
+            #   2) the stack does not yet exist, or
+            #   3) the stack exists and an 'on-updates' instruction was NOT provided, or
+            #   4) the stack exists and a value should be provided 'on-updates', then
+            # provide a value
+
+            if ( costing or 
+                 not stack_exists or 
+                 (stack_exists and 'on-updates' not in param_mapping[param_key]) or
+                 (stack_exists and 'on-updates' in param_mapping[param_key] and param_mapping[param_key]['on-updates'])
+               ):
+
+                stack_parameters = stack_parameters + [{"ParameterKey": param_mapping[param_key]['key'],
+                                                        "ParameterValue": param_mapping[param_key]['value']}]
+            else:
+                # use previous value
+                stack_parameters = stack_parameters + [{"ParameterKey": param_mapping[param_key]['key'],
+                                                        "UsePreviousValue": True }]
+
+    return stack_parameters
+
+def get_user_inputs(service_params):
+
+    stack_parameter = {}
+
+    # get inputs driven from service file
+    user_inputs = get_variable(service_params, 'user-inputs')
+
+    # get inputs driven from vpc preferences
+    if ('vpc-defaults' in service_params and 
+        'user-inputs' in service_params['vpc-defaults']):
+        user_inputs.update(service_params['vpc-defaults']['user-inputs'])
+    
+    return user_inputs
+
 def string_wizard(param_desc, help_msg, options, required, str_len=""):
 
     value = validated_input(param_desc,
@@ -195,7 +316,17 @@ def int_wizard(param_desc, help_msg, options, required, arg=""):
                             int_validation,
                             required)
 
-    return value    
+    return value
+
+def bool_wizard(param_desc, help_msg, options, required, arg=""):
+
+    value = validated_input(param_desc,
+                            help_msg, 
+                            ["True", "False"],
+                            bool_validation,
+                            required)
+
+    return value == "True"
 
 def array_wizard(param_desc, help_msg, options, required, layer=""):
       
@@ -223,10 +354,10 @@ def change_wizard(current_value, field_name):
         input = raw_input(msg)
 
         # validate the input
-        validation_failed = string_validation(input, options)
+        validation_failed = string_validation(input, options, False)
 
     # change iff 'y' was input
-    if input == 'y':
+    if (input and input == 'y'):
         change = True
 
     return change
@@ -245,17 +376,25 @@ def validated_input(param_desc, help_msg, options, function, required=True, opti
 
     if options:
 
-        input = raw_input("Press <enter> to see a list of availble options >> ")
-        print "Choices include: \n%s"%pp_list(options)
+        if len(options)>10:
+            input = raw_input("Press <enter> to see a list of availble options >> ")
+            print "Choices include: \n%s"%pp_list(options)
+        else:
+            print "Choices include: \n%s"%pp_short_list(options)
 
     while validation_failed:
 
-        input = raw_input("Please paste in one of the available options for %s >> "%param_desc)
+        if options:
+            input_msg = "Please paste in one of the available options for %s >> "%param_desc
+        else:
+            input_msg = "Please input a value for %s >> "%param_desc
+
+        input = raw_input(input_msg)
 
         input = input.strip("'")
 
         # validate the input
-        validation_failed = function(input, options, optional_arg)
+        validation_failed = function(input, options, required, optional_arg)
 
     return input
 
@@ -288,7 +427,7 @@ def validated_array_input(param_desc, help_msg, options, function, required=True
         if input:
             inputs.append(input)
         else:
-            validation_failed,inputs = function(inputs, options, optional_arg)
+            validation_failed,inputs = function(inputs, options, required, optional_arg)
 
     return inputs
 
@@ -310,7 +449,7 @@ def validated_array_input_old(param_desc, retry_msg, options, function):
         else:
             validation_failed,inputs = function(inputs, options,retry_msg)
 
-def string_validation(input,options,max_strlen=4000):
+def string_validation(input,options,required,max_strlen=4000):
 
     validation_failed = False
     if options:
@@ -328,12 +467,16 @@ def string_validation(input,options,max_strlen=4000):
             validation_failed = True
             retry_msg = "Input invalid (too long) - please input a string with <= %s chars"%max_strlen
 
+    if required and not input:
+        validation_failed = True
+        retry_msg = "Input required - please enter a value"
+
     if validation_failed:
         print retry_msg
 
     return validation_failed
 
-def int_validation(input,options,max_value=sys.maxint):
+def int_validation(input,options,required,max_value=sys.maxint):
 
     validation_failed = False
     if options:
@@ -346,7 +489,11 @@ def int_validation(input,options,max_value=sys.maxint):
             validation_failed = True
             retry_msg = "Input invalid - please input an int <= %s"%max_value
 
-    if not input.isdigit():
+    if required and not input:
+        validation_failed = True
+        retry_msg = "Input required - please enter a value"
+
+    if ( input and not input.isdigit() ):
         validation_failed = True
         retry_msg = "Input invalid - integers only"
         
@@ -355,7 +502,22 @@ def int_validation(input,options,max_value=sys.maxint):
 
     return validation_failed    
 
-def input_validation(input,options,retry_msg):
+def bool_validation(input,options,required,optional_arg=""):
+
+    validation_failed = (input not in ["True", "False"])
+
+    if required and not input:
+        retry_msg = "Input required - please enter a value"
+
+    elif validation_failed:
+        retry_msg = "Input invalid - True/False only"
+
+    if validation_failed:
+        print retry_msg
+
+    return validation_failed  
+
+def input_validation(input,options,required,retry_msg):
 
     # attempt to find the vpc input
     validation_failed = input not in options
@@ -365,7 +527,7 @@ def input_validation(input,options,retry_msg):
 
     return validation_failed
 
-def array_validation(inputs,options,arg=""):
+def array_validation(inputs,options,required,arg=""):
 
     validation_failed = len(Set(inputs) & Set(options)) != len(inputs)
 
@@ -514,8 +676,22 @@ def get_subnet_ids(params):
 
     return subnet_ids
 
+# get the id's of db subnet groups available in a given vpc and a given set of 
+# availability zones
+def get_db_subnet_group_ids(params):
+
+    vpc_id = get_variable(params,"vpc-id") 
+
+    rds = boto3.client('rds')
+    
+    subnets = rds.describe_db_subnet_groups()
+
+    db_subnet_group_ids = jmespath.search("DBSubnetGroups[?VpcId=='%s'].DBSubnetGroupName"%vpc_id,subnets)
+
+    return db_subnet_group_ids       
+
 # find the versions (tags) available for a container image
-def get_image_versions(params, image_name, registry_url='https://registry.hub.docker.com'):
+def get_image_versions(image_name, registry_url='https://registry.hub.docker.com'):
 
     # endpoint use to determine the latest version in the registry input for this app
     endpoint_uri = "/v2/repositories/%s/tags"%(image_name)
@@ -528,26 +704,38 @@ def get_image_versions(params, image_name, registry_url='https://registry.hub.do
 
     return versions
 
+# get all service-inputs collected from the user at service installation time.
+def get_stack_service_inputs( stack_name ):
+
+    decoded_dict = {}
+
+    stack_param_key = USER_INPUTS_PARAM_KEY
+
+    param_value = get_stack_param_value (stack_name , stack_param_key) 
+
+    if param_value:
+
+        # decode the params dict
+        decoded_dict = json.loads(param_value)
+
+    return decoded_dict 
+
 # get the value from a stack param that contains service-inputs collected
 # from the user at service installation time.
-# param_tuple should contain: [stack_param_key, yac_param_key]
-def get_stack_service_inputs_value( service_params , param_tuple ):
+def get_stack_service_inputs_value( stack_name , yac_param_key ):
 
     yac_param_value = ""
 
-    if len(param_tuple)==2:
+    stack_param_key = USER_INPUTS_PARAM_KEY
 
-        stack_param_key = param_tuple[0]
-        yac_param_key = param_tuple[1] 
+    param_value = get_stack_param_value (stack_name , stack_param_key) 
 
-        param_value = get_stack_param_value (service_params , stack_param_key) 
-        
-        if param_value:
+    if param_value:
 
-            # decode the params dict
-            decoded_dict = json.loads(param_value)
-            if yac_param_key in decoded_dict:
-                yac_param_value = get_variable(decoded_dict,yac_param_key)
+        # decode the params dict
+        decoded_dict = json.loads(param_value)
+        if yac_param_key in decoded_dict:
+            yac_param_value = get_variable(decoded_dict,yac_param_key)
 
     return yac_param_value 
 
@@ -558,6 +746,13 @@ def pp_list(list):
 
     return str
 
+def pp_short_list(list):
+    str = ""
+    for item in list:
+        str = str + ' %s'%item
+
+    return "( %s )"%str    
+
 def pp_help(help):
 
     if type(help)==list:
@@ -566,3 +761,21 @@ def pp_help(help):
         ret = str(help)
 
     return ret
+
+def validate_inputs_script(inputs_script, prefs_path):
+
+    script_path = os.path.join(prefs_path,inputs_script)
+
+    val_errors = {}
+
+    if os.path.exists(script_path):
+
+        naming_module = imp.load_source('yac.lib',script_path)
+
+        if 'get_value' not in dir(naming_module):
+            val_errors.update({"inputs-script": "inputs script %s lacks the get_value fxn"%inputs_script})
+
+    else:
+        val_errors = {"inputs-script": "inputs script %s doesn't exist"%inputs_script}
+
+    return val_errors  
