@@ -78,3 +78,24 @@ def set_namer(service_descriptor, servicefile_path, vpc_prefs):
 
     # save namer path to local variable
     set_local_value('yac_namer', service_namer_path)
+
+def validate_namer_script(namer_script, prefs_path):
+
+    val_errors = {}
+
+    script_path = os.path.join(prefs_path,namer_script)
+
+    if os.path.exists(script_path):
+
+        naming_module = imp.load_source('yac.lib',script_path)
+
+        if 'get_stack_name' not in dir(naming_module):
+            val_errors.update({"stack-namer": "namer script %s lacks the get_stack_name fxn"%namer_script})
+
+        if 'get_resource_name' not in dir(naming_module):
+            val_errors.update({"resource-namer": "namer script %s lacks the get_resource_name fxn"%namer_script})
+
+    else:
+        val_errors = {"namer-script": "namer script %s doesn't exist"%namer_script}
+
+    return val_errors    
