@@ -9,9 +9,7 @@ def get_stack_name( params ):
     delimitter = get_variable(params,'delimitter','-')
     
     name_parts = [get_variable(params,'prefix',''), 
-                    get_variable(params,'service-alias',''),
-                    get_variable(params,'suffix',''),
-                    get_variable(params,'env','') ]
+                    get_variable(params,'service-alias','')]
 
     # get rid of empty strings
     name_parts = filter(None,name_parts)
@@ -26,8 +24,6 @@ def get_resource_name(params, resource):
     delimitter = get_variable(params,'delimitter','-')
 
     name_parts = [get_variable(params,'service-alias',''),
-                  get_variable(params,'suffix',''), 
-                  get_variable(params,'env',''),
                   resource]
 
     # get rid of empty strings
