@@ -1,8 +1,9 @@
 #!/usr/bin/env python
-import os, json, urlparse, boto3, subprocess, shutil, jmespath
+import os, json, urlparse, boto3, subprocess, shutil, jmespath, sys, time
+import datetime as dt
 from botocore.exceptions import ClientError
 from sets import Set
-from yac.lib.file import FileError, file_in_registry, get_file_contents
+from yac.lib.file import FileError, file_in_registry, get_file_contents, localize_file, get_dump_path
 from yac.lib.template import apply_templates_in_file, apply_templates_in_dir
 from yac.lib.paths import get_config_path
 from yac.lib.intrinsic import apply_fxn
@@ -12,38 +13,26 @@ from yac.lib.naming import get_stack_name
 UPDATING = "Updating"
 BUILDING = "Building"
 
-def validate_services_consumed(services_consumed):
+STACK_STATES = ['CREATE_IN_PROGRESS','CREATE_FAILED','CREATE_COMPLETE','ROLLBACK_IN_PROGRESS',
+                'ROLLBACK_FAILED','ROLLBACK_COMPLETE','DELETE_IN_PROGRESS','DELETE_FAILED',
+                'DELETE_COMPLETE','UPDATE_IN_PROGRESS','UPDATE_COMPLETE_CLEANUP_IN_PROGRESS',
+                'UPDATE_COMPLETE','UPDATE_ROLLBACK_IN_PROGRESS','UPDATE_ROLLBACK_FAILED',
+                'UPDATE_ROLLBACK_COMPLETE_CLEANUP_IN_PROGRESS','UPDATE_ROLLBACK_COMPLETE']
 
-    valid_services_consumed = get_all_services_consumed()
+CREATE_IN_PROGRESS_STATE = 'CREATE_IN_PROGRESS'
 
-    validation_errors = {}
+NEWLY_CREATED_STATE = 'CREATE_COMPLETE'
 
-    unsupported_resources = list(Set(services_consumed) - Set(valid_services_consumed))
+UPDATE_COMPLETE_STATE = 'UPDATE_COMPLETE'
+UPDATE_COMPLETE_STATES = ['UPDATE_COMPLETE','UPDATE_ROLLBACK_COMPLETE']
 
-    if unsupported_resources:
-        validation_errors["unsupported-resources"] = unsupported_resources
+UPDATE_IN_PROGRESS_STATE = 'UPDATE_IN_PROGRESS'
 
-    return validation_errors
+UPDATABLE_STATES = ['CREATE_COMPLETE','UPDATE_COMPLETE','UPDATE_ROLLBACK_COMPLETE']
 
-def pretty_print_resources(resources):
+UNKNOWN_STATE = "unknown"
 
-    resource_map = {
-        "asg":   "  Auto-Scaling Group",
-        "efs":   "  Elastic File System (for shared home)",
-        "i-elb": "Internal ELB (intranet-facing)",
-        "e-elb": "External ELB (internet-facing)",
-        "rds":   "  Relational DB Service"
-    }
-    pp_resources = "\n"
-
-    for resource_key in resources:
-        pp_resources = pp_resources + '* %s: %s\n'%(resource_key,resource_map[resource_key])
-
-    return pp_resources
-
-def get_all_services_consumed():
-    # currently supported build-in services
-    return ['asg', 'efs', 'rds', 'e-elb', 'i-elb']
+NON_EXISTANT = "non-existant"
 
 # determine cost of a new stack
 def cost_stack( template_string="", stack_params = None ):
@@ -99,27 +88,63 @@ def update_stack( stack_name ,
 
     return response       
 
-def get_stack_templates(service_descriptor, vpc_prefs, service_parmeters):
+def get_stack_state( stack_name ):
+
+    client = boto3.client('cloudformation')
+
+    if stack_exists( stack_name ):
+
+        response = client.describe_stacks(StackName=stack_name)
+
+        states = jmespath.search("Stacks[*].StackStatus",response)
+
+        if len(states) == 1:
+            state = states[0]
+        else:
+            state = UNKNOWN
+
+    else:
+        state = NON_EXISTANT
+
+    return state 
+
+def get_stack_templates(service_descriptor, service_parmeters):
+
+    service_template = {}
+
+    # get the template from the service_descriptor
+    if 'stack-template' in service_descriptor:
+
+        # get the template from the descriptor
+        service_template_with_refs = service_descriptor['stack-template']
+
+        # get any resources that are to be excluded based on user input
+        resource_exclusions = get_variable(service_parmeters,'resource-exclusions',[])
+
+        for resource in resource_exclusions:
+            if resource in service_template_with_refs['Resources']:
+                print "removing the %s resource per requested exclusion ..."%resource
+                service_template_with_refs['Resources'].pop(resource) 
+
+        # render intrinsics
+        service_template = apply_fxn(service_template_with_refs, service_parmeters)
+
+    return service_template
+
+def get_stack_templates_old(service_descriptor, vpc_prefs, service_parmeters):
 
     # get the services this service consumes
     services_consumed = get_variable(service_descriptor,'services-consumed',[])
 
     # get the template files to use based on services specified
-    template_files = _get_stock_template_files(services_consumed, vpc_prefs)
-
-    # Add load-balancer-refs to service_parmeters based on elbs specified.
-    # The references are used by the ASG to understand which ELBs are incoming
-    load_balancer_refs = []
-    if ('i-elb' in services_consumed):
-        load_balancer_refs.append({"Ref": "InternalElb"})
-    if ('e-elb' in services_consumed):
-        load_balancer_refs.append({"Ref": "ExternalElb"})
-
-    set_variable(service_parmeters,"load-balancer-refs",load_balancer_refs) 
+    template_files = _get_stock_template_files(services_consumed, vpc_prefs) 
 
     # combine all resources into a single stack dictionary
     stack_template = {}
+
     for key in template_files.keys():
+
+        print "merging in the cf template in %s for the %s service ..."%(template_files[key],key)
         
         # pull template from file into a dictionary while also rendering any service 
         # parmeters into the dictionary
@@ -127,7 +152,8 @@ def get_stack_templates(service_descriptor, vpc_prefs, service_parmeters):
                                                         service_parmeters)
 
         # merge this resource into the stack definition
-        stack_template = update_template(stack_template,this_resource_template) 
+        stack_template = update_template(stack_template,this_resource_template)
+
 
     # next add any resources or resource changes specified in the service_descriptor
     if 'stack-template' in service_descriptor:
@@ -137,7 +163,7 @@ def get_stack_templates(service_descriptor, vpc_prefs, service_parmeters):
 
         # merge these resources into the stack definition
         stack_template = update_template(stack_template,service_template)
- 
+
 
     return stack_template
 
@@ -147,8 +173,8 @@ def _get_stock_template_files(services_consumed, vpc_prefs):
     vpc_prefs_template_map = {}
 
     # get template map provided in the vpc preferences
-    if 'services-consumed' in vpc_prefs:
-        vpc_prefs_template_map = vpc_prefs['services-consumed']
+    if 'service-templates' in vpc_prefs:
+        vpc_prefs_template_map = vpc_prefs['service-templates']
 
     # default templates supported
     template_map = {
@@ -290,18 +316,19 @@ def stack_exists(stack_name):
 
     try:
         response = client.describe_stacks(StackName=stack_name)
-        return len(response['Stacks'])>0
+        stack_count = len(response['Stacks'])
+        return stack_count>0
     except:
         return False
 
-def deploy_stack_files(service_descriptor, service_parmeters): 
+def deploy_stack_files(service_descriptor, service_parmeters, servicefile_path): 
 
     # deploy any individual files that the service needs before booting
     # (default to empty array in case variable is not defined)
     files_for_boot = get_variable(service_descriptor, 'deploy-for-boot', [], 'files')
 
     # render service parmeters in the files then deploy 'em
-    _load_files(files_for_boot, service_parmeters)
+    _load_files(files_for_boot, service_parmeters, servicefile_path)
 
     # deploy any directories that the service needs before booting
     dirs_for_boot = get_variable(service_descriptor, 'deploy-for-boot', [], 'directories')
@@ -310,9 +337,56 @@ def deploy_stack_files(service_descriptor, service_parmeters):
     # the directory structure to destination
     _load_dirs(dirs_for_boot, service_parmeters)
 
+
 # Render service parmeters into file body, then load files to destination
 # Only s3 destinations are currently supported.
-def _load_files(files, service_parmeters):
+def _load_files(files, service_parmeters, servicefile_path):
+
+    # assume the file path is relative to the location
+    # of the service descriptor file (just like Dockerfile!)
+    servicefile_path = get_variable(service_parmeters,"servicefile-path")
+
+    for this_ifile in files:
+
+        # render intrinsics in the file dictionary
+        this_file = apply_fxn(this_ifile, service_parmeters)      
+
+        # if necessary, localize file
+        localized_file = localize_file(this_file['src'], servicefile_path)
+
+        # render the file into the local 'dump' directory
+        rendered_file_path = get_dump_path(get_variable(service_parmeters,"service-name","unknown"))
+
+        if os.path.exists(localized_file):
+
+            # replace any service parmeters variables in the file body and return the 
+            # name+path of the "rendered" file
+            rendered_file = apply_templates_in_file(localized_file, service_parmeters, rendered_file_path)
+  
+            # if destination is s3 bucket
+            if (is_s3_destination(this_file['dest']) and rendered_file):
+
+                # copy rendered file to s3 destination 
+                cp_file_to_s3( rendered_file, this_file['dest'])
+
+            # if destination is another local file (mostly used for testing)
+            else:
+
+                # make sure destination directory exists
+                if not os.path.exists(os.path.dirname(this_file['dest'])):
+                    os.makedirs(os.path.dirname(this_file['dest']))
+
+                # copy rendered file to the destination
+                shutil.copy(rendered_file,this_file['dest']) 
+
+        else:
+
+            raise FileError( "%s file deploy was not performed. Source file is missing"%localized_file )
+
+
+# Render service parmeters into file body, then load files to destination
+# Only s3 destinations are currently supported.
+def _load_files_new(files, service_parmeters):
 
     # assume the file path is relative to the location
     # of the service descriptor file (just like Dockerfile!)
@@ -458,17 +532,10 @@ def sync_dir_to_s3(source_dir, destination_s3_url):
 # Get the ip addresses of ec2 instances in a stack identified by stack_name
 # with a name containing name_search_string.
 # If no name_search_string provided, returns IP addresses of all stack EC2 instances
-def get_ec2_ips( stack_name , name_search_string=""):
-
-    # get the stack
-    cloudformation = boto3.client('cloudformation')
-    stack = cloudformation.describe_stacks(StackName=stack_name)
-
-    # get the first stack's 'stack id'
-    stack_id = stack['Stacks'][0]['StackId']
+def get_ec2_ips( stack_name , name_search_string="", publicIP=False):
 
     # filters for stack_id in cloudformation tag
-    filters = [{'Name':"tag:aws:cloudformation:stack-id", 'Values' : [stack_id]}]
+    filters = [{'Name':"tag:aws:cloudformation:stack-name", 'Values' : [stack_name]}]
 
     # if specifying a Name, filter on that name
     if name_search_string:
@@ -476,27 +543,41 @@ def get_ec2_ips( stack_name , name_search_string=""):
 
     #pull response with instances matching filters
     client = boto3.client('ec2')
-    response = client.describe_instances(Filters=filters)
+    instances = client.describe_instances(Filters=filters)
 
-    # if there are Reservations pull instances out of response, else set
-    # instances to an empty list
-    if response['Reservations']:
-        instances = response['Reservations'][0]['Instances']
+    # get the private or public ip address of instances that are running
+    if publicIP:
+        ips = jmespath.search("Reservations[?Instances[?State.Name=='running']].Instances[*].PublicIpAddress", instances)        
     else:
-        instances = []
+        ips = jmespath.search("Reservations[?Instances[?State.Name=='running']].Instances[*].PrivateIpAddress", instances)
 
-    ips = []
+    # because the of the reservations in the outer layer, the search above will return 
+    # the IP array in a out array, e.g.
+    # [[ip1, ip2, etc.]]
 
-    #get ip for each instance
-    for instance in instances:
-        ips.append(instance['PrivateIpAddress'])
+    # we need to get rid of outer array
+    return ips[0]
 
-    return ips
+def get_ec2_sgs( stack_name , name_search_string=""):
+
+    # filters for stack_id in cloudformation tag
+    filters = [{'Name':"tag:aws:cloudformation:stack-name", 'Values' : [stack_name]}]
+
+    # if specifying a Name, filter on that name
+    if name_search_string:
+        filters.append({'Name':"tag:Name", 'Values' : [name_search_string]})
+
+    #pull response with instances matching filters
+    client = boto3.client('ec2')
+    sgs = client.describe_security_groups(Filters=filters)
+
+    # we need to get rid of outer array
+    return sgs['SecurityGroups']
 
 # get the RDS endpoints associated with a stack
 def get_rds_endpoints( stack_name ):
 
-    endpoint = {}
+    endpoints = []
 
     # get the resources associated with the stack
     cloudformation = boto3.client('cloudformation')
@@ -517,10 +598,113 @@ def get_rds_endpoints( stack_name ):
         client = boto3.client('rds')
         instances = client.describe_db_instances(DBInstanceIdentifier=rds_id)
 
-        endpoints = jmespath.search("DBInstances[*].Endpoint.{Address: Address, Port: Port}", instances)
+        # print "instances: %s"%json.dumps(instances,indent=2)
+
+        endpoints = jmespath.search("DBInstances[*].{Address: Endpoint.Address, Port: Endpoint.Port, Status: DBInstanceStatus}", instances)
 
     return endpoints
 
+# get the ECS services associated with a stack
+def get_ecs_service( stack_name , name_search_string):
+
+    to_ret={}
+
+    # get the resources associated with the stack
+    cloudformation = boto3.client('cloudformation')
+    resources = cloudformation.describe_stack_resources(StackName=stack_name)
+
+    service_id = ""
+    cluster_name = ""
+
+    if 'StackResources' in resources:
+
+        for resource in resources['StackResources']:
+
+            if resource['ResourceType'] == 'AWS::ECS::Service':
+
+                if name_search_string in resource['PhysicalResourceId']:
+                    service_id = resource['PhysicalResourceId']
+                    #print "ecs service: %s"%service_id
+
+            if resource['ResourceType'] == 'AWS::ECS::Cluster':
+
+                cluster_name = resource['PhysicalResourceId']
+                #print "ecs cluster: %s"%cluster_name                    
+
+    if service_id and cluster_name:
+
+        client = boto3.client('ecs')
+        services = client.describe_services(cluster=cluster_name, 
+                                            services=[service_id])
+
+        #print "ecs services: %s"%services
+        for service in services['services']:
+            if name_search_string in service['serviceName']:
+                to_ret = service
+                break
+
+    return to_ret       
+
+# get the ECS services associated with a stack
+def get_stack_elbs( stack_name , name_search_string):
+
+    to_ret={}
+
+    # get the resources associated with the stack
+    cloudformation = boto3.client('cloudformation')
+    resources = cloudformation.describe_stack_resources(StackName=stack_name)
+
+    elb_id = ""
+
+    if 'StackResources' in resources:
+
+        for resource in resources['StackResources']:
+
+            if resource['ResourceType'] == 'AWS::ElasticLoadBalancing::LoadBalancer':
+
+                if name_search_string in resource['PhysicalResourceId']:
+                    elb_id = resource['PhysicalResourceId']
+                    break
+                                    
+
+    if elb_id:
+
+        client = boto3.client('elb')
+        elbs = client.describe_load_balancers(LoadBalancerNames=[elb_id])
+
+        for elb in elbs['LoadBalancerDescriptions']:
+            if name_search_string in elb['LoadBalancerName']:
+                to_ret = elb
+                break
+    return to_ret    
+
+def get_rds_instance_ids( stack_name ):
+
+    instances_ids = []
+
+    # get the resources associated with the stack
+    cloudformation = boto3.client('cloudformation')
+    resources = cloudformation.describe_stack_resources(StackName=stack_name)
+
+    rds_id = ""
+
+    if 'StackResources' in resources:
+
+        for resource in resources['StackResources']:
+
+            if resource['ResourceType'] == 'AWS::RDS::DBInstance':
+
+                rds_id = resource['PhysicalResourceId']
+
+    if rds_id:
+
+        client = boto3.client('rds')
+        instances = client.describe_db_instances(DBInstanceIdentifier=rds_id)
+
+        instances_ids = jmespath.search("DBInstances[*].DBInstanceIdentifier", instances)
+
+    return instances_ids    
+
 # get the name of the elastic cache endpoint associated with a stack
 def get_cache_endpoint( stack_name ):
 
@@ -737,17 +921,7 @@ def get_elb_subnet_ids( load_balancer_name ):
         
     return subnet_ids
 
-def get_stack_s3_bucket(params):
-
-    return get_stack_param_value( params , "S3Bucket" )
-
-def get_stack_snapshot( params ):
-
-    return get_stack_param_value( params , "DBSnapshotId" )
-
-def get_stack_param_value( service_params , stack_param_name ):
-
-    stack_name = get_stack_name(service_params)
+def get_stack_param_value( stack_name , stack_param_name ):
     
     client = boto3.client('cloudformation')
 
@@ -789,4 +963,50 @@ def get_stack_tag_value( service_params , stack_tag_name ):
                 value = param['Value'] 
         
     return value
-   
\ No newline at end of file
+   
+def stop_service_blocking(stack_name, name_search_string):
+
+    ecs_service = get_ecs_service(stack_name, name_search_string)
+
+    if ecs_service and ecs_service['runningCount'] == 1:
+
+        print "Stopping the %s service ..."%ecs_service['serviceName']
+
+        # stop the service by setting the desired count to 0
+        client = boto3.client('ecs')
+        client.update_service(cluster=ecs_service['clusterArn'],
+                              service=ecs_service['serviceName'],
+                              desiredCount=0)
+
+        timer_start=dt.datetime.now()
+
+        # wait for the running count to change to "0"
+        while ( ecs_service['runningCount'] != 0 ):
+
+            now=dt.datetime.now()
+            elapsed_secs = (now-timer_start).seconds
+            sys.stdout.write('\r')
+            msg = ("After %s seconds, the service is still running ...")%(elapsed_secs)
+            sys.stdout.write(msg)
+            sys.stdout.flush()
+
+            # sleep for 10 seconds then check service state again
+            time.sleep(10)
+
+            # check the state of the service
+            ecs_service = get_ecs_service(stack_name, name_search_string)
+
+        # print an empty line to insert a cr
+        print ""  
+
+def start_service(stack_name, name_search_string):
+
+    ecs_service = get_ecs_service(stack_name, name_search_string)
+
+    if ecs_service and ecs_service['runningCount'] == 0:
+
+        # stop the service by setting the desired count to 0
+        client = boto3.client('ecs')
+        client.update_service(cluster=ecs_service['clusterArn'],
+                              service=ecs_service['serviceName'],
+                              desiredCount=1)            
\ No newline at end of file
