@@ -7,6 +7,8 @@ from yac.lib.validator import validate_dictionary
 from yac.lib.file_converter import convert_local_files, find_and_delete_remotes
 from yac.lib.variables import get_variable, set_variable
 from yac.lib.file import get_file_contents, create_customization_file, get_file_reg_key
+from yac.lib.naming import validate_namer_script
+from yac.lib.inputs import validate_inputs_script
 
 YAC_VPC_SUFFIX="-vpc"
 
@@ -18,11 +20,32 @@ SUPPORTED_KEYS=[ "prefs-repo",
                  "vpc-inputs",
                  "vpc-params",
                  "resource-namer",
-                 "stock-templates"]
+                 "service-inputs",
+                 "service-templates"]
 
-def validate_vpc_prefs(vpc_prefs):
+def validate_vpc_prefs(vpc_prefs, prefs_path = ""):
 
-    return validate_dictionary(vpc_prefs, SUPPORTED_KEYS, REQUIRED_FIELDS)
+    prefs_errors = validate_dictionary(vpc_prefs, SUPPORTED_KEYS, REQUIRED_FIELDS)
+
+    # if a namer is specified as part of the vpc preferences
+    namer_script = get_variable(vpc_prefs, "resource-namer","")
+    
+    if namer_script:
+        namer_errors = validate_namer_script(namer_script, prefs_path)
+
+        prefs_errors.update(namer_errors)
+
+    # if an inputs script is specified as part of the vpc preferences
+    if "service-inputs" in vpc_prefs:
+        
+        inputs_script = get_variable( vpc_prefs, "service-inputs","")
+        
+        if inputs_script:            
+            inputs_errors = validate_inputs_script(inputs_script, prefs_path)
+
+            prefs_errors.update(inputs_errors)        
+
+    return prefs_errors
 
 def get_vpc_prefs():
 
@@ -98,9 +121,37 @@ def get_vpc_prefs_from_registry(vpc_def_registry_key):
         vpc_prefs_str = get_remote_value('%s-vpc'%vpc_def_registry_key)
 
         if vpc_prefs_str:
+
             # convert back to dictionary
             vpc_prefs = json.loads(vpc_prefs_str)
 
+            # see if an inputs script is specified
+            vpc_inputs_yac_path = get_variable(vpc_prefs,'service-inputs',"")
+            
+            print 'inputs path: %s'%vpc_inputs_yac_path
+            
+            if vpc_inputs_yac_path:
+
+                # save the file under yac/lib/customizations
+                inputs_file_local_path = create_customization_file(vpc_inputs_yac_path)
+
+                # save the  new path back to the preferences
+                vpc_prefs['service-inputs']['value'] = inputs_file_local_path
+
+            # If the vpc prefs references a namer script, then we need to save it
+            # as a yac customization so it can be executed against any Scriptfile.
+
+            # see if a namer script is specified
+            vpc_namer_yac_path = get_variable(vpc_prefs,'resource-namer',"")
+
+            if vpc_namer_yac_path:
+
+                # save the file as a yac-local customization
+                namer_file_local_path = create_customization_file(vpc_namer_yac_path)
+
+                # change file reference so that instrincs recognizes it as part of the yac sources
+                set_variable(vpc_prefs,'resource-namer', namer_file_local_path)
+
     return vpc_prefs
 
 def get_all_vpc_def_keys():
@@ -132,7 +183,7 @@ def get_vpc_prefs_from_local_file(vpc_prefs_file):
     # as a yac customization so it can be executed against any Scriptfile.
 
     # see if an inputs script is specified
-    vpc_inputs_script_path = get_variable(vpc_prefs,'vpc-inputs',"")
+    vpc_inputs_script_path = get_variable(vpc_prefs,'service-inputs',"")
     
     if vpc_inputs_script_path:
 
@@ -144,8 +195,8 @@ def get_vpc_prefs_from_local_file(vpc_prefs_file):
         # save the file under yac/lib/customizations
         inputs_file_yac_path = create_customization_file(yac_file_key,prefs_file_contents)
 
-        # save the file as a yac-local customization
-        set_variable(vpc_prefs,'vpc-inputs',inputs_file_yac_path)
+        # save the  new path back to the preferences
+        vpc_prefs['service-inputs']['value'] = inputs_file_yac_path
 
     # If the vpc prefs references a namer script, then we need to save it
     # as a yac customization so it can be executed against any Scriptfile.
@@ -166,6 +217,7 @@ def get_vpc_prefs_from_local_file(vpc_prefs_file):
         # change file reference so that instrincs recognizes it as part of the yac sources
         set_variable(vpc_prefs,'resource-namer', namer_file_yac_path)
 
+
     return vpc_prefs 
 
 # get the subnets to use for a given vpc and a given set of 
